@@ -160,9 +160,6 @@ static const GtkTargetEntry target_table[] = {
 	{ "_NETSCAPE_URL", 0, 1 }
 };
 
-static void totem_playlist_class_init (TotemPlaylistClass *class);
-static void totem_playlist_init       (TotemPlaylist      *playlist);
-
 static void init_treeview (GtkWidget *treeview, TotemPlaylist *playlist);
 
 #define totem_playlist_unset_playing(x) totem_playlist_set_playing(x, TOTEM_PLAYLIST_STATUS_NONE)
@@ -1661,7 +1658,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 	g_return_val_if_fail (mrl != NULL, FALSE);
 
-	if (display_name == NULL)
+	if (display_name == NULL || *display_name == '\0')
 		filename_for_display = totem_playlist_mrl_to_title (mrl);
 	else
 		filename_for_display = g_strdup (display_name);
