@@ -50,7 +50,7 @@ totem_playing_dvd (const char *uri)
 }
 
 static void
-totem_ensure_dot_dir (const char *path)
+totem_ensure_dir (const char *path)
 {
 	if (g_file_test (path, G_FILE_TEST_IS_DIR) != FALSE)
 		return;
@@ -64,7 +64,7 @@ totem_dot_dir (void)
 	static char *totem_dir = NULL;
 
 	if (totem_dir != NULL) {
-		totem_ensure_dot_dir (totem_dir);
+		totem_ensure_dir (totem_dir);
 		return totem_dir;
 	}
 
@@ -72,7 +72,26 @@ totem_dot_dir (void)
 				      "totem",
 				      NULL);
 
-	totem_ensure_dot_dir (totem_dir);
+	totem_ensure_dir (totem_dir);
+
+	return (const char *)totem_dir;
+}
+
+const char *
+totem_data_dot_dir (void)
+{
+	static char *totem_dir = NULL;
+
+	if (totem_dir != NULL) {
+		totem_ensure_dir (totem_dir);
+		return totem_dir;
+	}
+
+	totem_dir = g_build_filename (g_get_user_data_dir (),
+				      "totem",
+				      NULL);
+
+	totem_ensure_dir (totem_dir);
 
 	return (const char *)totem_dir;
 }
@@ -151,7 +170,7 @@ totem_get_mount_for_media (const char *uri)
 	return ret;
 }
 
-static gboolean
+gboolean
 totem_is_special_mrl (const char *uri)
 {
 	GMount *mount;
@@ -241,6 +260,21 @@ static const char subtitle_ext[][4] = {
 	"ass"
 };
 
+gboolean
+totem_uri_is_subtitle (const char *uri)
+{
+	guint len, i;
+
+	len = strlen (uri);
+	if (len < 4 || uri[len - 4] != '.')
+		return FALSE;
+	for (i = 0; i < G_N_ELEMENTS (subtitle_ext); i++) {
+		if (g_str_has_suffix (uri, subtitle_ext[i]) != FALSE)
+			return TRUE;
+	}
+	return FALSE;
+}
+
 static inline gboolean
 totem_uri_exists (const char *uri)
 {
@@ -258,8 +292,9 @@ totem_uri_exists (const char *uri)
 static char *
 totem_uri_get_subtitle_for_uri (const char *uri)
 {
-	char *subtitle, *subtitle_ext_upper;
-	guint len, i, suffix;
+	char *subtitle;
+	guint len, i;
+	gint suffix;
 
 	/* Find the filename suffix delimiter */
 	len = strlen (uri);
@@ -285,9 +320,10 @@ totem_uri_get_subtitle_for_uri (const char *uri)
 	g_return_val_if_fail (subtitle != NULL, NULL);
 	g_strlcpy (subtitle, uri, suffix + 4 + 1);
 	g_strlcpy (subtitle + suffix, ".???", 5);
-	
+
 	/* Search for any files with one of our known subtitle extensions */
 	for (i = 0; i < G_N_ELEMENTS (subtitle_ext) ; i++) {
+		char *subtitle_ext_upper;
 		memcpy (subtitle + suffix + 1, subtitle_ext[i], 3);
 
 		if (totem_uri_exists (subtitle))
@@ -310,18 +346,18 @@ totem_uri_get_subtitle_in_subdir (GFile *file, const char *subdir)
 {
 	char *filename, *subtitle, *full_path_str;
 	GFile *parent, *full_path, *directory;
-  
+
 	/* Get the sibling directory @subdir of the file @file */
 	parent = g_file_get_parent (file);
 	directory = g_file_get_child (parent, subdir);
 	g_object_unref (parent);
-  
+
 	/* Get the file of the same name as @file in the @subdir directory */
 	filename = g_file_get_basename (file);
 	full_path = g_file_get_child (directory, filename);
 	g_object_unref (directory);
-  	g_free (filename);
-  
+	g_free (filename);
+
 	/* Get the subtitles from that URI */
 	full_path_str = g_file_get_uri (full_path);
 	g_object_unref (full_path);
@@ -331,6 +367,32 @@ totem_uri_get_subtitle_in_subdir (GFile *file, const char *subdir)
 	return subtitle;
 }
 
+static char *
+totem_uri_get_cached_subtitle_for_uri (const char *uri)
+{
+	char *basename, *fake_filename, *fake_uri, *ret;
+
+	basename = g_path_get_basename (uri);
+	if (basename == NULL || strcmp (basename, ".") == 0) {
+		g_free (basename);
+		return NULL;
+	}
+
+	fake_filename = g_build_filename (g_get_user_cache_dir (),
+				"totem",
+				"subtitles",
+				basename,
+				NULL);
+	g_free (basename);
+	fake_uri = g_filename_to_uri (fake_filename, NULL, NULL);
+	g_free (fake_filename);
+
+	ret = totem_uri_get_subtitle_for_uri (fake_uri);
+	g_free (fake_uri);
+
+	return ret;
+}
+
 char *
 totem_uri_get_subtitle_uri (const char *uri)
 {
@@ -351,6 +413,13 @@ totem_uri_get_subtitle_uri (const char *uri)
 		return NULL;
 	}
 
+	/* Try in the cached subtitles directory */
+	subtitle = totem_uri_get_cached_subtitle_for_uri (uri);
+	if (subtitle != NULL) {
+		g_object_unref (file);
+		return subtitle;
+	}
+
 	/* Try in the current directory */
 	subtitle = totem_uri_get_subtitle_for_uri (uri);
 	if (subtitle != NULL) {
@@ -483,7 +552,7 @@ totem_add_subtitle (GtkWindow *parent, const char *path)
 	char *subtitle = NULL;
 	gboolean set_folder;
 
-	fs = gtk_file_chooser_dialog_new (_("Select text subtitle"), 
+	fs = gtk_file_chooser_dialog_new (_("Select Text Subtitles"), 
 					  parent,
 					  GTK_FILE_CHOOSER_ACTION_OPEN,
 					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -497,16 +566,20 @@ totem_add_subtitle (GtkWindow *parent, const char *path)
 
 	conf = gconf_client_get_default ();
 	set_folder = TRUE;
-	if (path != NULL) {
+
+	new_path = gconf_client_get_string (conf, "/apps/totem/open_path", NULL);
+	if (new_path != NULL && *new_path != '\0') {
 		set_folder = gtk_file_chooser_set_current_folder_uri
-			(GTK_FILE_CHOOSER (fs), path);
-	} else {
-		new_path = gconf_client_get_string (conf, "/apps/totem/open_path", NULL);
-		if (new_path != NULL && *new_path != '\0') {
+			(GTK_FILE_CHOOSER (fs), new_path);
+	}
+	g_free (new_path);
+
+	if (path != NULL) {
+		if (set_folder == FALSE) {
 			set_folder = gtk_file_chooser_set_current_folder_uri
-				(GTK_FILE_CHOOSER (fs), new_path);
+				(GTK_FILE_CHOOSER (fs), path);
 		}
-		g_free (new_path);
+		gtk_file_chooser_add_shortcut_folder_uri (GTK_FILE_CHOOSER (fs), path, NULL);
 	}
 	
 	if (set_folder == FALSE) {
